from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *
from Products.ImageRepository.config import *

from Products.ATContentTypes.atct import ATBTreeFolderSchema
from Products.ATContentTypes.atct import ATCTBTreeFolder
from Products.ATContentTypes.interfaces import IATBTreeFolder
try:
    from Products.ATContentTypes.content.base import updateActions
except ImportError:
    # updateActions no longer exists in Plone 3
    def updateActions(*args):
        return ()

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import View, ModifyPortalContent, ManagePortal

from Products.CMFPlone.interfaces.Translatable import ITranslatable


def flatten(ltypes=(list, tuple), *args):
    for arg in args:
        if isinstance(arg, ltypes):
            for i in arg:
                for l in flatten(i):
                    yield l
        else:
            yield arg

def updateInterfaces(*args):
    return tuple(x for x in flatten(args) if x is not ITranslatable)


ImageRepositorySchema = ATBTreeFolderSchema.copy()

class ImageRepository(ATCTBTreeFolder):
    """A repository to store images."""

    content_icon   = 'imagerepository_icon.gif'
    meta_type      = 'ImageRepository'
    archetype_name = 'Image Repository'
    immediate_view = 'imagerepository_view'
    default_view   = 'imagerepository_view'
    suppl_views    = ()
    typeDescription= __doc__
    assocMimetypes = ()
    assocFileExt   = ()
    cmf_edit_kws   = ()

    schema = ImageRepositorySchema

    security = ClassSecurityInfo()

    actions = updateActions(ATCTBTreeFolder,
        ({
            'id'          : 'edit_keywords',
            'name'        : 'Tags',
            'action'      : 'string:${object_url}/imagerepository_tagging',
            'permissions' : (ModifyPortalContent,),
         },
         {
            'id'          : 'register_in_kupu',
            'name'        : 'Register repository in Kupu',
            'action'      : 'string:${object_url}/registerInKupu',
            'category'    : 'object_buttons',
            'condition'   : 'not:object/isRegisteredInKupu',
            'permissions' : (ManagePortal,),
         },
        )
    )

    __implements__ = updateInterfaces(ATCTBTreeFolder.__implements__, IATBTreeFolder)

    security.declarePrivate('at_post_create_script')
    def at_post_create_script(self):
        # disallow the adding of any type
        self.setConstrainTypesMode(1)
        self.setLocallyAllowedTypes(['Image'])
        self.setImmediatelyAddableTypes(['Image'])

    security.declareProtected(ManagePortal, 'registerInKupu')
    def registerInKupu(self):
        """Registeres this repository as a library in Kupu."""
        request = self.REQUEST
        response = request.RESPONSE
        if self.isRegisteredInKupu():
            message = "Already registered in Kupu."
        else:
            kupuTool = getToolByName(self, 'kupu_library_tool', None)
            if kupuTool is not None:
                portal_url = getToolByName(self, 'portal_url')
                library_ids = [x['id'] for x in kupuTool._libraries]
                repository_path = self.getPhysicalPath()
                portal_path = portal_url.getPhysicalPath()
                sub_path = repository_path[len(portal_path)-1:]
                path = "/".join(sub_path)
                name = "-".join(sub_path)
                if name not in library_ids:
                    kupuTool.addLibrary(name,
                                        'string:%s' % self.pretty_title_or_id(),
                                        'string:${portal_url}/%s/imagerepository_collection.xml' % path,
                                        'string:${portal_url}/%s/imagerepository_collection.xml' % path,
                                        'string:${portal_url}/imagerepository_icon.gif')
                    message = "Registered repository in Kupu."
            else:
                message = "Kupu is not installed."
        response.redirect(self.absolute_url()+'/view?portal_status_message='+message)

    security.declareProtected(View, 'isRegisteredInKupu')
    def isRegisteredInKupu(self):
        """Checks whether this repository as registered as a library in Kupu."""
        kupuTool = getToolByName(self, 'kupu_library_tool', None)
        if kupuTool is not None:
            portal_url = getToolByName(self, 'portal_url')
            library_ids = [x['id'] for x in kupuTool._libraries]
            repository_path = self.getPhysicalPath()
            portal_path = portal_url.getPhysicalPath()
            sub_path = repository_path[len(portal_path)-1:]
            path = "/".join(sub_path)
            name = "-".join(sub_path)
            return (name in library_ids)

registerType(ImageRepository, PROJECTNAME)
